Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;
USE work.pkg_081006.all; 

-------------------------------------------------------------------- 
-- Description:
-- This module implements a dual (ping-ponged) shift register for
-- receiving serial data from the AD7625, 16 Bit Analog to Digital
-- converter (ADC).  One shift register accepts the serial stream at the
-- same moment the other shift register is holding the last transmitted
-- value for the next stage of data processing.  
--
-- The shift register operation is controlled with a 5-bit counter.  
-- The counter's lower 4-bits count the 16 serial data bits shifted into 
-- the registers.  The 5th bit controls which of the two shift registers 
-- is receiving the serial stream and which is tied to the parallel data 
-- output of the module.
--
-- It is important that this module come out of reset and be ready to 
-- count the ADC Clocks before the starts sending clock and data transitions.

entity AD7625_SHIFT_REG_v5 is
	port(
		 clk           : in std_logic; -- global Logic Clock
		 reset_n       : in std_logic; -- global Logic Reset
		 ADC_reset_n	: in std_logic; -- adc specific reset
		 adc_srdata    : in std_logic; -- AD7625 signal Data 
		 adc_srclk     : in std_logic; -- AD7625 signal DCO  
		 
		 adc_data      : out std_logic_vector (15 downto 0);
		 adc_data_strb : out std_logic; -- parallel data and strobe out
		 sel_bit			: out std_logic
		);
end AD7625_SHIFT_REG_v5;
	
architecture main of AD7625_SHIFT_REG_v5 is
	-- Shift Register declarations -----------------------------------
	signal sr1         : std_logic_vector(15 downto 0); --
	signal sr2         : std_logic_vector(15 downto 0);
	signal ena1        : std_logic;
	signal ena2        : std_logic;
	signal strobe      : std_logic;
	
	signal q				: std_logic_vector(2 downto 0); -- for LED with ena1
	signal r				: std_logic_vector(2 downto 0); -- for LED with ena2
	
	-- Counter/Timer declarations ------------------------------------
	signal srcount     : std_logic_vector (8 downto 0);
	
--===begin===begin===begin===begin===begin===begin===begin===begin===
begin -- main --------------------------------------------------------

-- Shift Registers ---------------------------------------------------
   shift_reg1: 
	process (adc_srclk, ADC_reset_n, ena1, adc_srdata, sr1)
      begin
		if(ADC_reset_n = '0') then 
			sr1 <= (others => '0');
		elsif rising_edge(adc_srclk) then
			if (ena1 = '1') then 
				for i in 0 to 14 loop
					sr1(i+1) <= sr1(i);
				end loop;
				sr1(0) <= adc_srdata;
			end if;
      end if;
    end process;
    
   shift_reg2: 
	process (adc_srclk, ADC_reset_n, ena2, adc_srdata, sr2)
      begin
		if(ADC_reset_n = '0') then 
			sr2 <= (others => '0');
		elsif rising_edge(adc_srclk) then
			if (ena2 = '1') then 
				for i in 0 to 14 loop
					sr2(i+1) <= sr2(i);
				end loop;
				sr2(0) <= adc_srdata;
			end if;
       end if;
    end process;
	 
------------------------------------------------------------------------
-- Counter -------------------------------------------------------------
	srcnt: 
	process (adc_srclk, ADC_reset_n, srcount) -- bit counter for shift register
	begin
		if(ADC_reset_n = '0') then 					srcount <= (others => '0');
		elsif rising_edge(adc_srclk) then	srcount <= srcount + 1;
		end if;
	end process;
				 
	ena1 		<= not srcount(4); -- This is the 5th bit of the counter
	ena2 		<= srcount(4);
	sel_bit 	<= srcount(4);

-- Leading Edge Detectors for ena signals -------------------------------
	sync_ena: 
	process(clk, reset_n, ena1, ena2, q, r)
	begin
		if   (reset_n = '0') then	q <= (others => '0');
											r <= (others => '0');
		elsif rising_edge(clk) then
											q(0) <= ena1;
											q(1) <= q(0);
											q(2) <= q(1);
											r(0) <= ena2;
											r(1) <= r(0);
											r(2) <= r(1);
				adc_data_strb <= (q(1) and not q(2)) or (r(1) and not r(2));
		end if;
	end process;
	
			             
-- Syncronized Output Mux ------------------------------------------------
	sr_sm_outputs: process (clk, reset_n, q, sr2)
	begin
		if   (reset_n = '0') then	adc_data <= sr2;
		elsif rising_edge(clk) then
			if (q(0) = '1') then 	adc_data <= sr2;
			else                 	adc_data <= sr1;
			end if;
		end if;		
	end process; -- sclk_sm_outputs	
--------------------------------------------------------------------------


end main;

