--------------------------------------------------------------------
Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;
USE work.pkg_081006.all;  -- my definitions and entities

-------------------------------------------------------------------- 
-- 5/21/2015  This module manages the external interface bus.  This 
--            bus connects the FPGA to FRAM Memory and the external
--            interface.  It assumes the FPGA is expected to be the 
--            Bus Master and the other two end points memory when then
--            Bus_Mstr_FPGA input is pulled HIGH.
-- 
--            If Bus_Mstr_FPGA is pulled LOW it is assumed that the 
--            external interface is the Bus Master.
--
--				  IO line C[5..1] change roles depending on the Bus Master
--            configuration.
--
--            When Bus Master is External Interface
--            C[1] = Memory Read
--            C[2] = Memory Write Strobe
--            C[3] = FPGA chip select
--            C[4] = FRAM chip select
--            C[5] = reserved
--
--            When Bus Master is FPGA
--            C[1] = Ext. Memory Read
--            C[2] = Ext. Memory Write Strobe
--            C[3] = Ext. Memory chip select 1
--            C[4] = Ext. Memory chip select 2
--            C[5] = reserved
--
--            The FPGA has its own set of FRAM memory control lines.
--
--            ALSO ...
--            There is a mux that channels diagnostic words out to then
--            external memory interface.  It is expected that this mode
--            is used to send internal words, such as ADC data, to a 
--            logic analyzer connection, or provide additional digital 
--            outputs.


entity DiagnosticsMux is
	port(
			clk   			: in std_logic;
			nreset			: in std_logic;
			Bus_Mstr_FPGA 	: in std_logic; -- selects FPGA as Bus Master when '1'
			Bus_Mstr_Ext	: in std_logic; -- selects Ext. Interface as Bus Master
													 -- Otherwise Diagnostic data is put on bus
			DiagSel			: in std_logic_vector(3 downto 0);
			
			-- FPGA Bus Master Control Lines In --------------
			FPGA_Mem_Addr		: in std_logic_vector(19 downto 0);
			FPGA_Mem_Data		: in std_logic_vector(15 downto 0);
			FPGA_Mem_Rd			: in std_logic;
			FPGA_Mem_Wr			: in std_logic;
			FPGA_Mem_CE			: in std_logic;
			FPGA_Mem_FRAM_CS	: in std_logic;
			FPGA_Mem_EXT_CS1	: in std_logic;
			FPGA_Mem_EXT_CS2	: in std_logic;
			
			-- External Interface Bus Master Control Lines In -----
			EXT_Mem_Rd_C1		: in std_logic;
			EXT_Mem_Wr_C2 		: in std_logic;
			EXT_Mem_CE_C3		: in std_logic;
			EXT_Mem_A16_C4		: in std_logic;
			
			-- Internal Data Mux'd to External Interface --------
			Data_1	: in std_logic_vector(15 downto 0);
			Strb_1	: in std_logic;
			Data_2	: in std_logic_vector(15 downto 0);
			Strb_2	: in std_logic;
			Data_3	: in std_logic_vector(15 downto 0);
			Strb_3	: in std_logic;
			Data_4	: in std_logic_vector(15 downto 0);
			Strb_4	: in std_logic;
			Data_5	: in std_logic_vector(15 downto 0);
			Strb_5	: in std_logic;
			
			-- Address and Data Lines --------------------------
			Addr_Out		: out std_logic_vector(17 downto 0);
			Data_Out		: out std_logic_vector(15 downto 0);
			Drive_Data	: out std_logic;
			Drive_Addr	: out std_logic;
			
			-- FRAM Memory Control Lines -----------------------
			FRAM_WEn		: out std_logic;
			FRAM_OEn		: out std_logic;
			FRAM_CEn		: out std_logic;
			
			-- External Memory Control Lines (FPGA Bus Mstr.) --
			EXT_Rd_Out_C1 		: out std_logic;
			EXT_Wr_Out_C2 		: out std_logic;
			EXT_CS1_Out_C3 	: out std_logic;
			EXT_CS2_Out_C4 	: out std_logic;
			EXT_reserved_C5	: out std_logic;
			
			-- FPGA Memory Control Lines (EXT Bus Mstr.) --
			FPGA_Rd		: out std_logic;
			FPGA_Wr		: out std_logic;
			FPGA_CE		: out std_logic;
			
			-- Transceiver Control Signals --
			PA_DIR1		: out std_logic; -- drives C[5..1] from FPGA to Ext Intf
			PA_OEn1		: out std_logic; -- enables C[5..1] transceiver output either dir
			
			PA_DIR2		: out std_logic; -- drives Data[7..0] from FPGA to Ext Intf
			PA_OEn2		: out std_logic; -- enable Data Bus transceiver output either dir
			
			PC_XMTn		: out std_logic; -- drive Data[15..8] from FPGA to Ext Intf
			PC_RCVn		: out std_logic; -- drive Data[15..8] from Ext Intf to FPGA 
			
			PB_DIR1		: out std_logic; -- drives Addr[7..0] from FPGA to Ext Intf
			PB_OEn1		: out std_logic; -- enables Addr[7..0] transceiver output either dir
			
			PB_DIR2		: out std_logic; -- drives Addr[15..8] from FPGA to Ext Intf
			PB_OEn2		: out std_logic  -- enables Addr[15..8] transceiver output either dir
			
						
		);
end DiagnosticsMux;
	
architecture main of DiagnosticsMux is
	
	signal timer			: std_logic_vector(3 downto 0);
	signal ena_timer		: std_logic;
	signal default_clk	: std_logic;	

	signal data_strobe 	: std_logic;
	
	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(rst_timer, run_timer, strobe_clk);
	signal CS, NS  : STATE_TYPE;
	
--===================================================================
-- 			B	E	G	I	N	
--===================================================================
begin 
EXT_reserved_C5 <= '1';
		
DIAG_MUX_PROCESS:
process(clk, nreset, DiagSel, 
		FPGA_Mem_Addr, FPGA_Mem_Data, FPGA_Mem_Rd, FPGA_Mem_Wr, FPGA_Mem_CE,
		FPGA_Mem_FRAM_CS, FPGA_Mem_EXT_CS1, FPGA_Mem_EXT_CS2,
		EXT_Mem_A16_C4, EXT_Mem_CE_C3, EXT_Mem_Wr_C2, EXT_Mem_Rd_C1,
		Data_1, Strb_1, Data_2, Strb_2, Data_3, Strb_3, Data_4, Strb_4, Data_5, Strb_5, 
		data_strobe, default_clk, Bus_Mstr_FPGA, Bus_Mstr_Ext)
begin
	if   (Bus_Mstr_FPGA = '1') then

		Addr_Out				<= FPGA_Mem_Addr(17 downto 0);
		Data_Out				<= FPGA_Mem_Data;
		Drive_Data 			<= NOT FPGA_Mem_Rd;
		Drive_Addr			<= '1';
		
		-- Control Signals to FRAM -------
		FRAM_CEn 			<= NOT(FPGA_Mem_CE AND FPGA_Mem_FRAM_CS);
		FRAM_WEn				<= NOT(FPGA_Mem_Wr AND FPGA_Mem_FRAM_CS);
		FRAM_OEn				<= NOT(FPGA_Mem_Rd AND FPGA_Mem_FRAM_CS);
		
		-- Control Signals to External Interface ----
		EXT_Rd_Out_C1 		<= FPGA_Mem_Rd;
		EXT_Wr_Out_C2 		<= FPGA_Mem_Wr;
		EXT_CS1_Out_C3		<= FPGA_Mem_EXT_CS1;
		EXT_CS2_Out_C4 	<= FPGA_Mem_EXT_CS2;
		--EXT_BusPermit_C5 	<= '0'; 

		-- Signals only active with Ext. Interface Bus Master ----
		FPGA_Rd				<= '0';
		FPGA_Wr				<= '0';
		FPGA_CE				<= '0';
                     	
		-- Transceiver Control Signals --
		if (FPGA_Mem_Addr(19) = '0') then -- Not addressing Ext. Intf --
			-- The direction is always local bus to external bus and
			-- the transceivers are enabled to be able to monitor the bus.
			
			PA_DIR1	<= '1';	 -- drives C[5..1] from FPGA to Ext Intf
			PA_OEn1	<= '0';	 -- enables C[5..1] transceiver output either dir
			
			PA_DIR2	<= '1';	 -- drives Data[7..0] from FPGA to Ext Intf
			PA_OEn2	<= '0';	 -- enable Data Bus transceiver output either dir
			
			PC_XMTn	<= '0';	 -- drive Data[15..8] from FPGA to Ext Intf
			PC_RCVn	<= '1';	 -- disables Data[15..8] from Ext Intf to FPGA 
			
			PB_DIR1	<= '1';	 -- drives Addr[7..0] from FPGA to Ext Intf
			PB_OEn1	<= '0';	 -- enables Addr[7..0] transceiver output either dir
			       
			PB_DIR2	<= '1';	 -- drives Addr[15..8] from FPGA to Ext Intf
			PB_OEn2	<= '0';	 -- enables Addr[15..8] transceiver output either dir
			
		else -- addressing the Ext. Interface --
			-- The direction of the transceivers must change if reading from 
			-- Ext. Interface.  The FRAM will not be driving the bus at these
			-- addresses above 0x80000 (A(19)=1).
		
			PA_DIR1	<= '1';	 -- drives C[5..1] from FPGA to Ext Intf
			PA_OEn1	<= '0';	 -- enables C[5..1] transceiver output either dir
			
			PA_DIR2	<= NOT FPGA_Mem_Rd;	 -- if not reading drives Data[7..0] from FPGA to Ext Intf
			PA_OEn2	<= '0';	 -- enable Data Bus transceiver output either dir
			
			PC_XMTn	<= NOT(NOT FPGA_Mem_Rd); -- make active (low) if not reading	
			PC_RCVn	<= NOT(    FPGA_Mem_Rd); -- make active (low) if reading	 
			
			PB_DIR1	<= '1';	 -- drives Addr[7..0] from FPGA to Ext Intf
			PB_OEn1	<= '0';	 -- enables Addr[7..0] transceiver output either dir
			       
			PB_DIR2	<= '1';	 -- drives Addr[15..8] from FPGA to Ext Intf
			PB_OEn2	<= '0';	 -- enables Addr[15..8] transceiver output either dir
			
		end if;
				
		data_strobe <= '0'; -- just to avoid latching in this state
	
	elsif  (Bus_Mstr_Ext = '1') then
	
			Addr_Out				<= FPGA_Mem_Addr(17 downto 0); -- Tri-Stated Before Bus
			Data_Out				<= FPGA_Mem_Data; 
			Drive_Data 			<= EXT_Mem_Rd_C1; -- drive data if reading
			Drive_Addr			<= '0';
			
			FRAM_CEn 			<= NOT(EXT_Mem_CE_C3 AND EXT_Mem_A16_C4);
			FRAM_WEn				<= NOT(EXT_Mem_Wr_C2 AND EXT_Mem_A16_C4);
			FRAM_OEn				<= NOT(EXT_Mem_Rd_C1 AND EXT_Mem_A16_C4);
			
			EXT_Rd_Out_C1 		<= '0'; -- Tri-Stated Before Bus
			EXT_Wr_Out_C2 		<= '0'; -- Tri-Stated Before Bus
			EXT_CS1_Out_C3		<= '0'; -- Tri-Stated Before Bus
			EXT_CS2_Out_C4 	<= '0'; -- Tri-Stated Before Bus
         --EXT_BusPermit_C5 	<= '1'; -- always driven -- not available !!
	
			FPGA_CE				<= (EXT_Mem_CE_C3 AND NOT EXT_Mem_A16_C4);
			FPGA_Wr				<= (EXT_Mem_Wr_C2 AND NOT EXT_Mem_A16_C4);
			FPGA_Rd				<= (EXT_Mem_Rd_C1 AND NOT EXT_Mem_A16_C4);
 
			-- Transceiver Control Signals --
			PA_DIR1	<= '0';	 -- drives C[5..1] from Ext Intf to FPGA
			PA_OEn1	<= '0';	 -- enables C[5..1] transceiver output either dir
			
			PA_DIR2	<= EXT_Mem_Rd_C1; -- if reading drives Data[7..0] from FPGA to Ext Intf
			PA_OEn2	<= '0';	 		   -- enable Data Bus transceiver output either dir
			
			PC_XMTn	<= NOT(    EXT_Mem_Rd_C1);	 -- make active (low) if reading
			PC_RCVn	<= NOT(NOT EXT_Mem_Rd_C1);	 -- make active (low) if not reading	 
			
			PB_DIR1	<= '0';	 -- drives Addr[7..0] from Ext Intf to FPGA 
			PB_OEn1	<= '0';	 -- enables Addr[7..0] transceiver output either dir
			       
			PB_DIR2	<= '0';	 -- drives Addr[15..8] from Ext Intf to FPGA 
			PB_OEn2	<= '0';	 -- enables Addr[15..8] transceiver output either dir
			
		data_strobe <= '0'; -- just to avoid latching in this state
								
	else	-- Diagnostic Output by default --
			Addr_Out				<= FPGA_Mem_Addr(17 downto 0);
			Drive_Data 			<= '1';
			Drive_Addr			<= '1';
		
			-- Control Signals to FRAM -------
			FRAM_CEn 			<= '1';
			FRAM_WEn				<= '1';
			FRAM_OEn				<= '1';
			
			-- Control Signals to External Interface ----
			EXT_Rd_Out_C1 		<= '0';
			EXT_Wr_Out_C2 		<= data_strobe;
			EXT_CS1_Out_C3		<= '0';
			EXT_CS2_Out_C4 	<= '0';
         -- EXT_BusPermit_C5 	<= '0';  -- not available !!

			-- Signals only active with Ext. Interface Bus Master ----
			FPGA_Rd				<= '0';
			FPGA_Wr				<= '0';
			FPGA_CE				<= '0';
			
			PA_DIR1	<= '1';	 -- drives C[5..1] from FPGA to Ext Intf
			PA_OEn1	<= '0';	 -- enables C[5..1] transceiver output either dir
			
			PA_DIR2	<= '1';	 -- drives Data[7..0] from FPGA to Ext Intf
			PA_OEn2	<= '0';	 -- enable Data Bus transceiver output either dir
			
			PC_XMTn	<= '0'; -- make active (low) since we are not reading	
			PC_RCVn	<= '1'; -- make inactive (high) since we are not reading	
			
			PB_DIR1	<= '1';	 -- drives Addr[7..0] from FPGA to Ext Intf
			PB_OEn1	<= '0';	 -- enables Addr[7..0] transceiver output either dir
			       
			PB_DIR2	<= '1';	 -- drives Addr[15..8] from FPGA to Ext Intf
			PB_OEn2	<= '0';	 -- enables Addr[15..8] transceiver output either dir
			
		
		case DiagSel is
			when X"1" =>
				Data_Out				<= Data_1;
				data_strobe			<= Strb_1;
				
			when X"2" =>
				Data_Out				<= Data_2;
				data_strobe			<= Strb_2;
				
			when X"3" =>
				Data_Out				<= Data_3;
				data_strobe			<= Strb_3;
				
			when X"4" =>
				Data_Out				<= Data_4;
				data_strobe			<= Strb_4;
								
			when X"5" =>
				Data_Out				<= Data_5;
				data_strobe			<= Strb_5;

 			when others =>
				Data_Out				<= X"BAD1";
				data_strobe			<= default_clk;

		end case;
	end if;

end process;	

------------
TIMER_PROCESS:
process(clk, nreset, ena_timer)
begin
	if   (nreset = '0') then timer <= (others => '0');
	elsif rising_edge(clk) then
		if   (ena_timer = '1') then timer <= timer + 1;
		else						timer <= (others => '0');
		end if;
	end if;
end process;

------------
TIMER_STATE_MACHINE:
process (clk, nreset)	
begin
	if(nreset='0')              then CS <= rst_timer;
	elsif(clk'event and clk='1')then CS <= NS;
	end if;
end process; 

sm_combinatorial: process (CS, timer)
begin
	ena_timer 	<= '0';
	default_clk <= '0';
	
	case CS is
		when rst_timer   =>				NS <= run_timer;
			ena_timer 	<= '0';		 
		-----
		when run_timer =>			
			ena_timer 	<= '1';
			
			if   (timer = X"C") then  	NS <= strobe_clk;
			else								NS <= run_timer;
			end if;			
		-----
		when strobe_clk =>          	NS <= run_timer;
			ena_timer 	<= '0';
			default_clk <= '1';		
		-----	
		when others =>						NS <= rst_timer;
	end case;
end process;

end main;

