Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;

USE work.pkg_081006.all; 

-------------------------------------------------------------------- 
-- Description: This version applies two "clk" cycles per "sclk".

entity LTC2641_Interface_v2 is
	port(
		 clk           : in std_logic; -- global Logic Clock
		 nreset        : in std_logic; -- global Logic Reset
		 dac_data      : in std_logic_vector(15 downto 0); 
		 data_strb     : in std_logic;   
		 
		 dac_ncs 		: out std_logic; 
		 dac_sclk 		: out std_logic;
		 dac_din	   	: out std_logic;
		 dac_ready		: out std_logic
		);
end LTC2641_Interface_v2;
	
architecture main of LTC2641_Interface_v2 is
	-- Shift Register declarations -----------------------------------
	signal sr         	: std_logic_vector(15 downto 0); --
	signal shift       	: std_logic;
	signal load			: std_logic;
	signal dac_cs		: std_logic;
	
	-- Process Trigger LED -------------------------------------------
	signal go	      	: std_logic;
	
	-- Counter/Timer declarations ------------------------------------
	signal nclr_cnt    : std_logic;
	signal incr_cnt	   : std_logic;
	signal clr_cnt	   : std_logic;
	signal srcount     : std_logic_vector (4 downto 0);
	
	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(idle, sclk_lo, sclk_hi);
	signal CS, NS  : STATE_TYPE;
	
--===begin===begin===begin===begin===begin===begin===begin===begin===
begin -- main --------------------------------------------------------

-- Update Date Trigger ---------------------------------------------
STROBE_LED: led
port map(sig_in => data_strb, clk => clk, clrn => nreset, pls_out => go);
	
-- Shift Registers ---------------------------------------------------
    shift_reg: process (clk, nreset, shift, load)
	begin
		if(nreset = '0') 		then sr <= (others => '0');
		elsif rising_edge(clk) then
			if (shift = '1') then 
			  for i in 0 to 14 	loop sr(i+1) <= sr(i); end loop;
			elsif(load = '1') 	then sr <= dac_data;
			end if;
		end if;
    end process;
    
	dac_din 	<= sr(15);
	dac_ncs		<= NOT dac_cs;
	
-- Shift Counter -------------------------------------------------------
SHIFT_COUNT: process (clk, nreset, clr_cnt, incr_cnt) -- bit counter for shift register 
	begin
		if(nreset = '0') then srcount <= (others => '0');

		elsif rising_edge(clk)  then 
			if    (clr_cnt = '1')  then srcount <= (others => '0');
			elsif (incr_cnt = '1') then srcount <= srcount + 1;
			else					    srcount <= srcount;
			end if;
		end if;
	end process;
				 
------------------------------------------------------------------------	
-- State Machine -------------------------------------------------------
	sm_reg: process (clk, nreset)	
	begin
		if(nreset = '0')            then CS <= idle;
		elsif(clk'event and clk='1')then CS <= NS;
		end if;
	end process; -- sm_reg
	
	sm: process (CS, go, srcount)
	begin
		-- default signal values
		shift  		<= '0';
		load		<= '0';
		incr_cnt  	<= '0';
		clr_cnt		<= '0';
		dac_cs		<= '0';
		dac_ready	<= '0';
		dac_sclk	<= '0';
		-------------------------
		case CS is
			when idle   	=>
				dac_ready	<= '1';
				load 		<= '1';
				clr_cnt		<= '1';
				if(go = '1')		then	NS <= sclk_lo;
				else    					NS <= idle;
				end if;	
						    
			when sclk_lo 	=>				NS <= sclk_hi;
				dac_sclk	<= '0';
				dac_cs		<= '1';

			when sclk_hi 	=>				
				dac_sclk	<= '1';
				dac_cs		<= '1';
				shift 		<= '1';
				incr_cnt  	<= '1';
				
				if(srcount = X"f")	then	NS <= idle;
				else						NS <= sclk_lo;
				end if;
				
			when others 	=>				NS <= idle;
		end case; 				
	end process; -- sm
	
end main;

