Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;

-- My Definitions --------------------------------
USE work.pkg_081006.all; 

------------------------------------------------------
-- THIS VERSION EXPECTS clk_25MHz = 25 MHz
-- SEE clkdiv counter.

------------------------------------------------------
-- The DPOT Control Word is defined as follows
-- 
-- |15 ... 14|13 ... 8| 7| 6 ... 3  |    2    |  1  |  0  |
-- |Enable=11| NSteps | -| DPOT_Addr|Ena Store| Dir | Step|
--
-- Enable Slot0 Cntrl 		<= DPOT_Cntrl_Word(15 downto 14);
-- Nsteps(5 downto 0) 		<= DPOT_Cntrl_Word(13 downto 8);
-- DPOT_Addr(4 downto 0) 	<= DPOT_Cntrl_Word(6 downto 3);
-- Ena_Store					<= DPOT_Cntrl_Word(2);
-- Direction					<= DPOT_Cntrl_Word(1);
-- Step							<= DPOT_Cntrl_Word(0);

-- Step bit must be written Low then High in order to initiate
-- the process.

entity MCP4021_Interface4 is
	port(
--			clk					: in std_logic;
			clk_25MHz			: in std_logic;
			reset_n				: in std_logic;
			DPOT_Cntrl_Word	: in std_logic_vector(15 downto 0);
			
			Enc_Ena_Store		: in std_logic;
			Enc_DPOT_Addr		: in std_logic_vector(3 downto 0);
			Enc_Direction		: in std_logic;
			Enc_Step				: in std_logic;
			Enc_Input_Enable	: in std_logic_vector(1 downto 0);
			
			DPOT_Addr			: out std_logic_vector(3 downto 0);
			CS_Strobe_Out		: out std_logic;
			Step_Pulse_Out		: out std_logic;
			Enable_DPOT_Adj	: out std_logic;
			
			go_out				: out std_logic;
			incr_count_test	: out std_logic;
			clock_out			: out std_logic;
			step_cmd_out		: out std_logic;
			clr_start_out		: out std_logic;
			stepcnt_incr_out	: out std_logic;
			stepcnt_clrn_out	: out std_logic
		);
end MCP4021_Interface4;

architecture main of MCP4021_Interface4 is

signal Direction		: std_logic;
signal Step				: std_logic;
signal Nsteps			: std_logic_vector(5 downto 0);
signal Ena_Store		: std_logic;

-- Signals to latch and syncronize the inputs ----------------------------
signal led				: std_logic_vector(2 downto 0);
signal step_cmd		: std_logic;

signal dir				: std_logic;
signal go				: std_logic;
signal clr_start		: std_logic;

-- State Machine Outputs -------------------------------------------------
signal cs_strobe		: std_logic;
signal step_pulse		: std_logic;

-- Counter to implement chip select low to U_nD rising edge timing -------
signal count			: std_logic_vector(1 downto 0);
signal cnt_clrn		: std_logic;
signal cnt_done		: std_logic;

-- Clock Divider ---------------------------------------------------------
signal clkdiv			: std_logic_vector(4 downto 0);
signal clk_640ns		: std_logic;

-- Step Counter ----------------------------------------------------------
signal stepcnt			: std_logic_vector(7 downto 0);
signal stepcnt_clrn	: std_logic;
signal stepcnt_incr	: std_logic;


-- STATE_MACHINE --------------------------------------
type STATE_TYPE is
	(idle, setup_dir, hold_dir, setup_step, do_step, setup_delay, wr_wiper);
	signal CS, NS : STATE_TYPE;	

--========================================================
--                  B E G I N
--========================================================
begin

DPOT_CONTROL_SWITCH:
process( Enc_Input_Enable, DPOT_Cntrl_Word, 
			Enc_Ena_Store, Enc_DPOT_Addr, Enc_Direction, Enc_Step, 
			step_pulse, cs_strobe)
	begin
		if   (DPOT_Cntrl_Word(15 downto 14) = "11") then
			Nsteps(5 downto 0) 		<= DPOT_Cntrl_Word(13 downto 8);
			DPOT_Addr(3 downto 0) 	<= DPOT_Cntrl_Word(6 downto 3);
			Ena_Store					<= DPOT_Cntrl_Word(2);
			Direction					<= DPOT_Cntrl_Word(1);
			Step							<= DPOT_Cntrl_Word(0);
			Step_Pulse_Out  			<=	step_pulse;	
			CS_Strobe_Out				<= cs_strobe;
			Enable_DPOT_Adj			<= '1';
			
		elsif(Enc_Input_Enable = "11") then
			Nsteps(5 downto 0) 		<= "000001";
			DPOT_Addr(3 downto 0) 	<= NOT Enc_DPOT_Addr;	-- bits need invert	
			Ena_Store					<= Enc_Ena_Store;		
			Direction					<= Enc_Direction;		
			Step							<= Enc_Step;				
			Step_Pulse_Out  			<=	step_pulse;	
			CS_Strobe_Out				<= cs_strobe;
			Enable_DPOT_Adj			<= '1';
			
		else
			Nsteps(5 downto 0) 		<= "000001";
			DPOT_Addr(3 downto 0) 	<= "0000";
			Ena_Store					<= '0';
			Direction					<= '0';
			Step							<= '0';
			Step_Pulse_Out  			<=	'0';	
			CS_Strobe_Out				<= '1';
			Enable_DPOT_Adj			<= '0';
			
		end if;		
	end process;

TRIGGER_LED: -- Detect leading edge of Step. Step pulse might be long.
process(clk_25MHz, reset_n, Step)
begin
	if   (reset_n = '0') 	then led <= "000";
	elsif rising_edge(clk_25MHz)	then
		led(0) <= Step;
		led(1) <= led(0);
		led(2) <= led(1);
	end if;
end process;

step_cmd <= led(1) and not led(2);

--------------------------------------------------------------------------------
LATCH_DIRECTION: -- Latch Direction at the step_pulse.
process(clk_25MHz, reset_n, step_cmd)
begin
	if (reset_n = '0') 	   	then dir  <= '0';
	elsif rising_edge(clk_25MHz) 		then
		if   (step_cmd = '1')	then dir <= Direction;
		end if;	
	end if;
end process;

--------------------------------------------------------------------------------
COMMAND_GO: -- Hold the step command until state machine starts.
process(clk_25MHz, reset_n, clr_start, step_cmd)
begin
	if (reset_n = '0') 			then go <= '0';
	elsif rising_edge(clk_25MHz) then
		if   (step_cmd  = '1')	then go <= '1';			
		elsif(clr_start = '1')	then go <= '0';			
		end if;	
	end if;
end process;

---------------------------------------------------------------------------------
CLOCK_DIVIDER: process(clk_25MHz, reset_n)
begin
		if(reset_n = '0') then clkdiv <= "00000";
		elsif(rising_edge(clk_25MHz)) then 
			clkdiv <= clkdiv + 1;
			clk_640ns <= clkdiv(4);
		end if;
end process;

---------------------------------------------------------------------------------
SETUP_DELAY_TIMER:
process(clk_640ns, reset_n, cnt_clrn)
begin
	if (reset_n = '0') OR (cnt_clrn = '0') then count <= "00";
	elsif(rising_edge(clk_640ns)) then 
		count <= count + 1;
		if(count = "11") then 	cnt_done <= '1';
		else							cnt_done <= '0';
		end if;
	end if;
end process;

---------------------------------------------------------------------------------
--STEP_COUNTER: counter -- Count number of dpot steps taken
--generic map(6)
--port map(	clk => clk_640ns,
--				clrn => stepcnt_clrn,
--				ena  => stepcnt_incr,
--				load => '0',
--				updn => '1',
--				ld_data  => (others => '0'),
--				flag_cnt => Nsteps,
--				flag_out => stepcnt_done,
--				cnt_out  => open 
--			);

ALT_STEP_COUNTER: 
process(clk_640ns, reset_n, stepcnt_clrn, stepcnt_incr)
begin
	if   (reset_n = '0') then stepcnt <= (others => '0');
	elsif rising_edge(clk_640ns) then
		if   (stepcnt_clrn = '0') then stepcnt <= (others => '0');
		elsif(stepcnt_incr = '1') then stepcnt <= stepcnt + 1;
		end if;
	end if;
end process;

--=======================================================================
-- STATE MACHINE  
--=======================================================================
SM_REG: 
process(clk_640ns, reset_n)
begin
	if(reset_n = '0')				   then CS <= idle;
	elsif(rising_edge(clk_640ns))	then CS <= NS;
	end if;
end process;

SM_NEXT_STATE: 
--(idle, setup_dir, hold_dir, setup_step, do_step, setup_delay, wr_wiper);
process(CS, go, dir, cnt_done, Ena_Store, stepcnt, Nsteps)
begin
	-- DEFAULT VALUES ----------------
	clr_start	 <= '0';
	
	cnt_clrn	 	 <= '0';
	
	stepcnt_incr <= '0';
	stepcnt_clrn <= '1';
	
	cs_strobe	 <= '1';
	step_pulse	 <= '0';
	
	case CS is
		when idle => 	
			if(go = '1')	then					NS <= setup_dir;
			else										NS <= idle;
			end if;

			stepcnt_clrn <= '0';
			-------------------------------------------
			cs_strobe	<= '1'; 	step_pulse	<= dir;
			-------------------------------------------
		
		when setup_dir =>							NS <= hold_dir;
			clr_start	 <= '1';
			-------------------------------------------
			cs_strobe	<= '1';		step_pulse	<= dir;
			-------------------------------------------
		
		when hold_dir =>							NS <= setup_step;
			-------------------------------------------
			cs_strobe	<= '0';		step_pulse	<= dir;
			-------------------------------------------
		
		when setup_step =>
			cnt_clrn	<= '1';
			if(cnt_done = '1') then 			NS <= do_step;
			else										NS <= setup_step;
			end if;
			--------------------------------------------
			cs_strobe	<= '0';	 	step_pulse	<= '0';
			--------------------------------------------
	                          
		when do_step =>							NS <= setup_delay;
			stepcnt_incr <= '1';
			-------------------------------------------
			cs_strobe	<= '0';		step_pulse	<= '1';
			-------------------------------------------
		
		when setup_delay =>			
			if   (stepcnt = Nsteps) then 	NS <= wr_wiper;
				step_pulse  <= dir XOR Ena_Store;
			else										NS <= do_step;
				step_pulse	<= '0';
			end if;
			--------------------
			cs_strobe	<= '0';	
			--------------------
		
		when wr_wiper =>							NS <= idle;
			-------------------------------------------
			cs_strobe	<= '1';						
			step_pulse  <= dir XOR Ena_Store;
			-------------------------------------------
			
		when others =>              			NS <= idle;
	end case;
end process;

go_out				<= go;
incr_count_test	<= stepcnt_incr;
clock_out			<= clk_640ns;
		
step_cmd_out		<= step_cmd;
clr_start_out		<= clr_start;
stepcnt_incr_out	<= stepcnt_incr;
stepcnt_clrn_out  <= stepcnt_clrn;

end;----------------------------------------------------------