Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;

-- My Definitions --------------------------------------------------
USE work.pkg_081006.all;  -- my definitions and entities


-- Version 1 :  This module receives controls the Select Switches
--  in order to select the source of the control voltage at then
--  modules output.  If the Mod_Sel_Trig does not fire at the 
--  beginning of the cycle, the External 1 input is routed to 
--  the output and this module remains inactive.  

--  If Mod_Sel_Trig fires, the phase detector control signal 
--  generated by this module is routed to the output and a timer 
--  will time the cycle and send a cycle reset signal at the end.
--  The Hold for MIPL logic will be acitve.

entity Module_Select_Logic is
	port(clk        	: in std_logic;
		 nreset     	: in std_logic;
		 Mod_Sel_Trig 	: in std_logic;

 		 Beam_Cyc_Reset	: out std_logic;
 		 Module_Switch_Sel: out std_logic;
 		 Extern_Switch_Sel: out std_logic

		);
end Module_Select_Logic;
	
architecture main of Module_Select_Logic is

signal go				: std_logic;
signal mod_selected	: std_logic;

signal timer			: std_logic_vector(23 downto 0);
signal timer_ena		: std_logic;

signal cycle_reset	: std_logic;

	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(idle, module_selected, send_reset);
	signal CS, NS  : STATE_TYPE;

-- ## MAIN ##############################################################
begin 
EDGE_ON_MODULE_SELECT_TRIGGER: led
port map(sig_in => Mod_Sel_Trig, clk => clk, clrn => nreset, pls_out => go);

----------------------------------------------------------
SWITCH_CONTROL: process(clk, nreset, mod_selected)
begin
	if    (nreset = '0') then 				mod_selected <= '0';
	elsif rising_edge(clk) then 
		if   (go = '1') then 				mod_selected <= '1';
		elsif(cycle_reset = '1') then 	mod_selected <= '0';
		end if;
	end if;
end process; 

----------------------------------------------------------
PROCESS_TIMER: process(clk, nreset, timer_ena, timer)
begin
	if  (nreset = '0') then 		timer <= (others => '0');
	elsif rising_edge(clk) then
		if  (timer_ena = '0') then timer <= (others => '0');
		else								timer <= timer + 1;
		end if;
	end if;
end process;


----------------------------------------------------------	
sm_reg: process (clk, nreset)	
begin
	if(nreset='0')              then CS <= idle;
	elsif(clk'event and clk='1')then CS <= NS;
	end if;
end process; -- sm_reg

sm_combinatorial: 
process (CS, timer, go)
begin
	-- default outputs --
	timer_ena 			<= '1'; -- not enabled only when in idle state
	cycle_reset			<= '0';
	
	Module_Switch_Sel	<= '1';	-- if we leave "idle" this is the default
	Extern_Switch_Sel	<= '0';	-- if we leave "idle" this is the default
 
	case CS is
		when idle   =>
			Module_Switch_Sel	<= '0';	
			Extern_Switch_Sel	<= '1';
			timer_ena			<= '0';  
			
			if (go = '1') then 						NS <= module_selected;
			else											NS <= idle;
			end if;
			
		when module_selected =>	
									  
			if   (timer = X"30D400")     then	NS <= send_reset; -- time out at 40 ms
			else											NS <= module_selected;
			end if;
						
		when send_reset  =>							NS <= idle;
			cycle_reset	<= '1';
			
		when others =>									NS <= idle;
	end case; 
			
end process;

------------------------------------------------------------------------------------
-- OUTPUTS
	Beam_Cyc_Reset			<= cycle_reset;
	Module_Switch_Sel		<= mod_selected;
	Extern_Switch_Sel 	<= NOT mod_selected;


end main;