Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;

-- My Definitions --------------------------------------------------
USE work.pkg_081006.all;  -- my definitions and entities


-- Version 1 :  This module receives controls the Select Switches
--  in order to select the source of the control voltage at then
--  modules output.  If the Mod_Sel_Trig does not fire at the 
--  beginning of the cycle, the External 1 input is routed to 
--  the output and this module remains inactive.  

--  If Mod_Sel_Trig fires, the phase detector control signal 
--  generated by this module is routed to the output and a timer 
--  will time the cycle and send a cycle reset signal at the end.
--  The Hold for MIPL logic will be active.

entity Sequence_Controller is
	port(clk        		: in std_logic;
		 nreset     		: in std_logic;
		 Accel_Gate_In		: in std_logic;
		 Mod_Sel_Trig 		: in std_logic;
		 Hold_for_MIPL 	: in std_logic;
		 F1_to_F2_Delay	: in std_logic_vector(15 downto 0);
		 F2_to_Hold_Delay	: in std_logic_vector(15 downto 0);
		 Ena_BW_Control	: in std_logic;
 		 
 		 Module_Switch_Sel: out std_logic;
 		 Extern_Switch_Sel: out std_logic;
		 Filter_1_Trigger : out std_logic;
		 Filter_2_Trigger : out std_logic;
		 HOLD_Trigger		: out std_logic;
		 
		 MIPL_Gate			: out std_logic;
		 
		 Beam_Cyc_Reset	: out std_logic

		);
end Sequence_Controller;
	
architecture main of Sequence_Controller is

signal go				: std_logic;
signal start_hold		: std_logic;

signal timer			: std_logic_vector(23 downto 0);
signal timer_ena		: std_logic;

signal delay1			: std_logic_vector(23 downto 0);
signal delay2			: std_logic_vector(23 downto 0);

signal cycle_reset	: std_logic;
signal bw_cntrl_enabled : std_logic;

	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(idle, module_selected, is_BW_enabled, trig_f1_curve, pause1, trig_f2_curve, 
	  pause2, hold, wait_for_tmr, send_reset); -- timer is 40 ms until hold then timer is 6 ms.
	signal CS, NS  : STATE_TYPE;

-- ## MAIN ##############################################################
begin 

	MIPL_Gate <= Accel_Gate_In;

EDGE_ON_MODULE_SELECT_TRIGGER: led
port map(sig_in => Mod_Sel_Trig, clk => clk, clrn => nreset, pls_out => go);

--LEVEL_ACTIVATED:
--process(clk, nreset, Mod_Sel_Trig)
--begin
--	if  (nreset = '0') then 			go <= '0';
--	elsif rising_edge(clk) then
--		if  (Mod_Sel_Trig = '1') then go <= '1';
--		else									go <= '0';
--		end if;
--	end if;
--end process;


EDGE_ON_HOLD_FOR_MIPL: led
port map(sig_in => Hold_for_MIPL, clk => clk, clrn => nreset, pls_out => start_hold);

----------------------------------------------------------
PROCESS_TIMER: process(clk, nreset, timer_ena, timer)
begin
	if  (nreset = '0') then 		timer <= (others => '0');
	elsif rising_edge(clk) then
		if  (timer_ena = '0') then timer <= (others => '0');
		else								timer <= timer + 1;
		end if;
	end if;
end process;

----------------------------------------------------------
Latch_BW_Enable: process(clk, nreset, Ena_BW_Control, cycle_reset)
begin
	if  (nreset = '0') then 		bw_cntrl_enabled <= '0';
	elsif rising_edge(clk) then
		if   (cycle_reset = '1') 	 then bw_cntrl_enabled <= '0';
		elsif(Ena_BW_Control = '1') then bw_cntrl_enabled <= '1';
		end if;
	end if;
end process;

----------------------------------------------------------	
	delay1 <= X"00" & F1_to_F2_Delay(15 downto 0);
	delay2 <= X"00" & F2_to_Hold_Delay(15 downto 0);

----------------------------------------------------------	
sm_reg: process (clk, nreset)	
begin
	if(nreset='0')              then CS <= idle;
	elsif(clk'event and clk='1')then CS <= NS;
	end if;
end process; -- sm_reg

sm_combinatorial: 
process (CS, timer, go, start_hold, F1_to_F2_Delay, F2_to_Hold_Delay, 
			bw_cntrl_enabled, delay1, delay2)
begin
	-- default outputs --
	timer_ena 			<= '1'; -- not enabled only when in idle state
	cycle_reset			<= '0';
	
	Module_Switch_Sel	<= '1';	-- if we leave "idle" this is the default
	Extern_Switch_Sel	<= '0';	-- if we leave "idle" this is the default
	
	Filter_1_Trigger	<= '0';
 	Filter_2_Trigger	<= '0';
	HOLD_Trigger		<= '0';
	

	case CS is
		when idle   =>
			Module_Switch_Sel	<= '0';	-- The switches switch at the tclk 2ms before beam
			Extern_Switch_Sel	<= '1';
			timer_ena			<= '0';  
			
			if (go = '1') then 						NS <= module_selected;
			else											NS <= idle;
			end if;
			
		when module_selected =>	
									  
			if   (timer = X"30D400")     then	NS <= send_reset; -- time out at 40 ms
			elsif(start_hold = '1')		  then	NS <= is_BW_enabled;
			else											NS <= module_selected;
			end if;
		
		when is_BW_enabled =>
			timer_ena			<= '0'; -- clear timer
			if   (bw_cntrl_enabled = '1') then 	NS <= trig_f1_curve;
			else											NS <= pause2; -- skip to hold delay_length
			end if;
		
		when trig_f1_curve =>						NS <= pause1;
			timer_ena			<= '0'; -- clear timer
			Filter_1_Trigger	<= '1'; -- trigger the Filter 1 BW curve to start
			
		when pause1 =>
			if   (timer = X"075300")     then	NS <= send_reset; -- 6 ms timeout
			elsif(timer = delay1)     	  then	NS <= trig_f2_curve; 
			else											NS <= pause1;
			end if;
			
		when trig_f2_curve =>						NS <= pause2;
			timer_ena			<= '0'; -- clear timer
			Filter_2_Trigger	<= '1'; -- trigger the Filter 2 BW curve to start
				
		when pause2 =>
			if   (timer = X"075300")     then	NS <= send_reset; -- 6 ms timeout
			elsif(timer = delay2)     	  then	NS <= hold; 
			else											NS <= pause2;
			end if;

		when hold =>									NS <= wait_for_tmr;
			-- timer is not reset
			if   (F2_to_Hold_Delay = X"FFFF") then HOLD_Trigger <= '0';
			else												HOLD_Trigger <= '1';
			end if;
			
		when wait_for_tmr => 
			if   (timer = X"075300")     then	NS <= send_reset; -- 6 ms timeout
			else											NS <= pause2;
			end if;
			
		when send_reset  =>							NS <= idle;
			cycle_reset	<= '1';
			
		when others =>									NS <= idle;
	end case; 
			
end process;

	Beam_Cyc_Reset		<= cycle_reset;


end main;