Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;

-- My Definitions --------------------------------------------------
USE work.pkg_081006.all;  -- my definitions and entities


--Version 4 breaks the EXTERNAL / MADC1 Switch logic out from the 
-- Accel_PL_Ena and Start / Stop logic.

entity Start_Stop_Logic_v4 is
	port(clk        	: in std_logic;
		 nreset     	: in std_logic;
		 Beam_Gate		: in std_logic;
		 Accel_Gate		: in std_logic;
		 Mod_Sel_Trig 	: in std_logic;
		 Hold_for_MIPL	: in std_logic;
		 Accel_Polarity : in std_logic;
		 Ena_ZC_Start	: in std_logic;
		 Test_Gate		: in std_logic;
		 
		 Accel_PL_Ena		: out std_logic;
 		 HOLD_Acc_PD		: out std_logic;
 		 Zero_Cross_Trig	: out std_logic;		 
 		 Beam_Cyc_Reset		: out std_logic;
 		 Module_Switch_Sel	: out std_logic;
 		 Extern_Switch_Sel  : out std_logic

		);
end Start_Stop_Logic_v4;
	
architecture main of Start_Stop_Logic_v4 is

signal syncd_Accel_Gate	: std_logic;
signal syncd_Beam_Gate 	: std_logic;
signal mod_selected		: std_logic;

signal timer			: std_logic_vector(23 downto 0);
signal timer_ena		: std_logic;

signal go				: std_logic;
signal hold_for_nobeam 	: std_logic;
signal cycle_reset		: std_logic;

-- Zero Crossing Start Trigger Signals --
signal ena_zc_strt	: std_logic;
signal polarity		: std_logic;


	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(idle, module_selected, setup_zc, hold_off_zc, find_zc, get_gates, cycle_active, 
		hold_pd_out, no_beam, send_reset);
	signal CS, NS  : STATE_TYPE;

-- ## MAIN ##############################################################
begin 
EDGE_ON_MODULE_SELECT_TRIGGER: led
port map(sig_in => Mod_Sel_Trig, clk => clk, clrn => nreset, pls_out => go);

process(clk, nreset, Beam_Gate, Accel_Gate, Ena_ZC_Start, Accel_Polarity)
begin
	if (nreset = '0') then 
		syncd_Accel_Gate	<= '0';
		syncd_Beam_Gate		<= '0';
		ena_zc_strt			<= '0';
		polarity			<= '0';
	elsif rising_edge(clk) then
		syncd_Accel_Gate	<= Accel_Gate;
		syncd_Beam_Gate		<= Beam_Gate OR Test_Gate;
		ena_zc_strt			<= Ena_ZC_Start;
		polarity			<= Accel_Polarity;
	end if;
end process;

PROCESS_TIMER: process(clk, nreset, timer_ena, timer)
begin
	if  (nreset = '0') then 	timer <= (others => '0');
	elsif rising_edge(clk) then
		if  (timer_ena = '0') then 	timer <= (others => '0');
		else						timer <= timer + 1;
		end if;
	end if;
end process;

----------------------------------------------------------
SWITCH_CONTROL: process(clk, nreset, go)
begin
	if    (nreset = '0')          then mod_selected <= '0';
	elsif rising_edge(clk) then 
		if   (go = '1') 		  then mod_selected <= '1';
		elsif(cycle_reset = '1')  then mod_selected <= '0';
		end if;
	end if;
end process; 

	Module_Switch_Sel	<= mod_selected;
	Extern_Switch_Sel 	<= NOT mod_selected;


----------------------------------------------------------	
sm_reg: process (clk, nreset)	
begin
	if(nreset='0')              then CS <= idle;
	elsif(clk'event and clk='1')then CS <= NS;
	end if;
end process; -- sm_reg

sm_combinatorial: 
process (CS, timer, syncd_Beam_Gate, syncd_Accel_Gate, go, ena_zc_strt, polarity)
begin
	-- default outputs --
	timer_ena 		<= '1'; -- not enabled only when in idle state
	Zero_Cross_Trig <= '1';
	cycle_reset		<= '0';
	
	Accel_PL_Ena	<= '0';
	hold_for_nobeam	<= '0';
		
	case CS is
		when idle   =>
			timer_ena		<= '0';  
			if (go = '1') then 					NS <= module_selected;	
			else								NS <= idle;
			end if;

		when module_selected =>
			-- logic for zero crossing start
			if   (ena_zc_strt = '1') then
				if (syncd_Beam_Gate = '1') then NS <= setup_zc;	-- saw beam gate active
				elsif(timer = X"0C3500") then	NS <= send_reset; -- time out at 10 ms
				else							NS <= module_selected;
				end if;
			-- logic for beam gate start
			else								NS <= get_gates;
												timer_ena	<= '0';  
			end if;

		------------------------------------------------------------------------
		--------------------------------------------------------------------------
		when setup_zc =>			-- start will depend on Beam Gate and zero crossing 				
												NS <= hold_off_zc;
			timer_ena		<= '0';  
			
		when hold_off_zc  =>
			if   (syncd_Beam_Gate = '0') then	NS <= hold_pd_out; -- lost beam
			elsif(timer = X"0007D0") then		NS <= find_zc;     -- time out at 25 us
			else								NS <= hold_off_zc;
			end if;
		
		when find_zc  =>
			Zero_Cross_Trig <= '0';  -- will go high again at zero crossing
			if   (polarity = '1') then			NS <= cycle_active;
			elsif(timer = X"0C3500") then		NS <= send_reset; -- time out at 10 ms
			else								NS <= find_zc;
			end if;
			
		--------------------------------------------------------------------------
		--------------------------------------------------------------------------
		when get_gates  =>			-- start depends of Beam Gate and Accel PL Gate
									-- we hope to see beam at least 4 ms after Booster Cycle Trigger
			if (syncd_Beam_Gate = '1') AND (syncd_Accel_Gate = '1')
									 then		NS <= cycle_active; 
			elsif(timer = X"04E200") then		NS <= send_reset; -- 4 ms time out 
			else								NS <= get_gates;
			end if;

		--------------------------------------------------------------------------
		when cycle_active  =>
			Accel_PL_Ena		<= '1';
									  
			if   (syncd_Beam_Gate = '0') then	NS <= hold_pd_out; -- we expect beam for full cycle
			elsif(timer = X"30D400")     then	NS <= send_reset; -- time out at 40 ms
			else								NS <= cycle_active;
			end if;

		when hold_pd_out  =>					NS <= no_beam;
			Accel_PL_Ena		<= '1';
			hold_for_nobeam		<= '1'; -- hold the acceleration PD before indicating no beam
										-- check timer just in case of major coincidence
		when no_beam  =>
			Accel_PL_Ena		<= '0';
			hold_for_nobeam		<= '1'; -- hold the acceleration PD until reset
			if(timer = X"30D400")   then		NS <= send_reset; -- time out at 40 ms
										-- we reset to get ready for next cycle after 40 ms
			else								NS <= no_beam;
			end if;
			
		when send_reset  =>						NS <= idle;
			cycle_reset	<= '1';
			
		when others =>							NS <= idle;
	end case; 
			
end process;

------------------------------------------------------------------------------------
	Beam_Cyc_Reset	<= cycle_reset;
	HOLD_Acc_PD 	<= hold_for_nobeam OR Hold_for_MIPL;

end main;