Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;

-- My Definitions --------------------------------------------------
USE work.pkg_081006.all;  -- my definitions and entities


--Version 4 breaks the EXTERNAL / MADC1 Switch logic out from the 
-- Accel_PL_Ena and Start / Stop logic.

--Version 5 adds an adjustable delay to Hold_for_MIPL

--Version 6 is modified to accomodate the Zero Crossing Detection
--  of the Accel PD feedback signal AC structure.  The feedback 
--  signal has small (~20mV) pulses caused by the passing of the
--  beam Notch passing the beam pickup plates.  At this time we
--  are not trying to eliminate this AC structure but we do want
--  to hold the Accel PD at MIPL at the zero crossing of this AC
--  component.  This will hopefully reduce the disturbance to the
--  Booster beam when the Accel PD feedback is held.
--
--  The scheme is to receive the signal to Hold_for_MIPL and then
--  wait for the zero crossing indication before holding.  There is
--  a timer that starts at Hold_for_MIPL and times out after 5 us.
--  The Accel PD will be held if the zero crossing indication is not
--  seen after this time.
--
--  The Zero Crossing Start of Accel PD feature has been removed from
--  this version.

entity Start_Stop_Logic_v6 is
	port(clk        	: in std_logic;
		 nreset     	: in std_logic;
		 Beam_Gate		: in std_logic;
		 Accel_Gate		: in std_logic;
		 Mod_Sel_Trig 	: in std_logic;
		 Hold_for_MIPL	: in std_logic;
		 Test_Gate		: in std_logic; -- alternate for Beam_Gate
		 Hold_Delay		: in std_logic_vector(15 downto 0);
		 ZC_Trig			: in std_logic; 

		 Accel_PL_Ena		: out std_logic;
 		 HOLD_Acc_PD		: out std_logic;
 		 Beam_Cyc_Reset	: out std_logic;
 		 Module_Switch_Sel: out std_logic;
 		 Extern_Switch_Sel: out std_logic

		);
end Start_Stop_Logic_v6;
	
architecture main of Start_Stop_Logic_v6 is

signal syncd_Accel_Gate	: std_logic;
signal syncd_Beam_Gate 	: std_logic;
signal mod_selected		: std_logic;

signal timer			: std_logic_vector(23 downto 0);
signal timer_ena		: std_logic;

signal go				: std_logic;
signal hold_for_nobeam 	: std_logic;
signal cycle_reset		: std_logic;

-- Hold Delay Timer --------------------
signal dly_tmr		: std_logic_vector(19 downto 0);
signal syncd_Hold	: std_logic;
signal dlyd_Hold	: std_logic;

-- Zero Crossing Timer --------------------
signal zc_detected: std_logic; 
signal zc_timer	: std_logic_vector(11 downto 0);	
signal zc_tmr_ena	: std_logic;
signal zc_time_out: std_logic;

	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(idle, get_gates, cycle_active, hold_for_MIPL_requested, do_MIPL_hold, 
		hold_pd_out, no_beam, send_reset);
	signal CS, NS  : STATE_TYPE;

-- ## MAIN ##############################################################
begin 
EDGE_ON_MODULE_SELECT_TRIGGER: led
port map(sig_in => Mod_Sel_Trig, clk => clk, clrn => nreset, pls_out => go);

EDGE_ON_ZERO_CROSSING_TRIGGER: led
port map(sig_in => ZC_Trig, clk => clk, clrn => nreset, pls_out => zc_detected);

----------------------------------------------------------
SWITCH_CONTROL: process(clk, nreset, go)
begin
	if    (nreset = '0') then 				mod_selected <= '0';
	elsif rising_edge(clk) then 
		if   (go = '1') then 				mod_selected <= '1';
		elsif(cycle_reset = '1') then 	mod_selected <= '0';
		end if;
	end if;
end process; 

----------------------------------------------------------
SYNC_INPUTS: process(clk, nreset, Beam_Gate, Accel_Gate)
begin
	if (nreset = '0') then 
		syncd_Accel_Gate	<= '0';
		syncd_Beam_Gate		<= '0';
		syncd_Hold			<= '0';
	elsif rising_edge(clk) then
		syncd_Accel_Gate	<= Accel_Gate;
		syncd_Beam_Gate		<= Beam_Gate OR Test_Gate;
		syncd_Hold			<= Hold_for_MIPL;
	end if;
end process;

HOLD_DELAY_TIMER: process(clk, nreset, dly_tmr, syncd_Hold)
begin
	if  (nreset = '0') then 			dly_tmr <= (others => '0');
	elsif rising_edge(clk) then
		if  (syncd_Hold = '0') then 	dly_tmr <= (others => '0');
		else					 	    		dly_tmr <= dly_tmr + 1;
		end if;
	end if;
end process;

ZERO_CROSSING_TIMER: process(clk, nreset, zc_timer, zc_tmr_ena)
begin
	if  (nreset = '0') then 			zc_timer <= (others => '0');
	elsif rising_edge(clk) then
		if   (zc_tmr_ena = '0') then 	zc_timer <= (others => '0');
		elsif(zc_timer = X"FF0") then zc_timer <= zc_timer;
		else					 		      zc_timer <= zc_timer + 1;
		end if;
	end if;
	
	if   (zc_timer > X"190") then	zc_time_out <= '1'; -- 5 us
	else							zc_time_out <= '0';
	end if;
	
end process;

PROCESS_TIMER: process(clk, nreset, timer_ena, timer)
begin
	if  (nreset = '0') then 		timer <= (others => '0');
	elsif rising_edge(clk) then
		if  (timer_ena = '0') then timer <= (others => '0');
		else								timer <= timer + 1;
		end if;
	end if;
end process;


----------------------------------------------------------	
sm_reg: process (clk, nreset)	
begin
	if(nreset='0')              then CS <= idle;
	elsif(clk'event and clk='1')then CS <= NS;
	end if;
end process; -- sm_reg

sm_combinatorial: 
process (CS, timer, syncd_Beam_Gate, syncd_Accel_Gate, go, 
			dly_tmr, Hold_Delay, syncd_Hold, zc_detected, zc_time_out)
begin
	-- default outputs --
	timer_ena 			<= '1'; -- not enabled only when in idle state
	cycle_reset			<= '0';
	
	Accel_PL_Ena		<= '0';
	dlyd_Hold			<= '0';
	hold_for_nobeam		<= '0';
	zc_tmr_ena			<= '0';
		
	case CS is
		when idle   =>
			timer_ena		<= '0';  
			if (go = '1') then 						NS <= get_gates;
			else											NS <= idle;
			end if;
			
		--------------------------------------------------------------------------
		when get_gates  =>	-- start depends of Beam Gate and Accel PL Gate
									-- we hope to see beam at least 4 ms after Booster Cycle Trigger
			if (syncd_Beam_Gate = '1') AND (syncd_Accel_Gate = '1') then	
															NS <= cycle_active; 
			elsif(timer = X"04E200") then			NS <= send_reset; -- 4 ms time out 
			else											NS <= get_gates;
			end if;

		--------------------------------------------------------------------------
		when cycle_active  =>						-- dly_tmr does not run until receiving
			Accel_PL_Ena		<= '1';				-- MIPL_Triggered (Hold_for_MIPL)
									  
			if   (timer = X"30D400")     then			NS <= send_reset; -- time out at 40 ms
			elsif(dly_tmr > X"0" & Hold_Delay) then 	NS <= hold_for_MIPL_requested;
			else													NS <= cycle_active;
			end if;
			
		when hold_for_MIPL_requested =>			-- this logic is a hold over from ZC start mode
			Accel_PL_Ena		<= '1';				-- and does very little now except delay 5 us 
			zc_tmr_ena			<= '1';				-- before going to gate syncd_Hold to dlyd_Hold
			
			if   (timer = X"30D400") then			NS <= send_reset; -- time out at 40 ms
			elsif(zc_time_out = '0') then  				-- zc arrival has not timed out.
				if   (zc_detected = '1') then		NS <= do_MIPL_hold;
				else										NS <= hold_for_MIPL_requested;
				end if;
			else											NS <= do_MIPL_hold; -- timed out waiting for zc
			end if;
			
		when do_MIPL_hold =>
			Accel_PL_Ena		<= '1';
			dlyd_Hold 			<= syncd_Hold;
			
			if(timer = X"30D400")   then			NS <= send_reset; -- time out at 40 ms
			else											NS <= do_MIPL_hold;
			end if;

		-----------------------------------------------------------------------------------		
		when hold_pd_out  =>						NS <= no_beam;
			Accel_PL_Ena		<= '1';
			hold_for_nobeam	<= '1'; -- hold the acceleration PD before indicating no beam
										-- check timer just in case of major coincidence
		when no_beam  =>
			Accel_PL_Ena		<= '0';
			hold_for_nobeam	<= '1'; -- hold the acceleration PD until reset
			if(timer = X"30D400")   then			NS <= send_reset; -- time out at 40 ms
										-- we reset to get ready for next cycle after 40 ms
			else											NS <= no_beam;
			end if;
			
		when send_reset  =>							NS <= idle;
			cycle_reset	<= '1';
			
		when others =>									NS <= idle;
	end case; 
			
end process;

------------------------------------------------------------------------------------
-- OUTPUTS
	Beam_Cyc_Reset			<= cycle_reset;
	HOLD_Acc_PD 			<= hold_for_nobeam OR dlyd_Hold;
	Module_Switch_Sel		<= mod_selected;
	Extern_Switch_Sel 	<= NOT mod_selected;


end main;