Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;

-- My Definitions --------------------------------------------------
USE work.pkg_081006.all;  -- my definitions and entities

-- HOW DOES IT WORK ------------------------------------------------
-- If a signal is detected to be high (on a logic clk edge) then
-- we require it to be high for atleast 25 ns (2 clks) before setting
-- the output high.  The output will remain high for atleast 500ns
-- before testing the input again.  After 500ns we will constantly
-- check for the input to go low.  If found low for atleast 25ns
-- we will set the output low for atleast 500ns before testing
-- for the input to be high.


entity UP_DN_Filter_v4 is
	port(clk        	: in std_logic;
		 nreset     	: in std_logic;
		 Input			: in std_logic;
		 
		 Output			: out std_logic		 
		);
end UP_DN_Filter_v4;
	
architecture main of UP_DN_Filter_v4 is

signal inp			: std_logic;

signal timer		: std_logic_vector(15 downto 0);
signal timer_ena	: std_logic;

	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(idle, wait_for_high, set_high, input_high, test_high, 
	               wait_for_low, set_low, input_low, test_low);
	signal CS, NS  : STATE_TYPE;

-- ## MAIN ##############################################################
begin 

SYNCHRONIZE_INPUT:
process(clk, nreset, Input)
begin
	if   (nreset = '0') then 	inp <= '0';
	elsif rising_edge(clk) then	inp <= Input;
	end if;	
end process;

PROCESS_TIMER: process(clk, nreset, timer_ena, timer)
begin
	if  (nreset = '0') then 	timer <= (others => '0');
	elsif rising_edge(clk) then
		if  (timer_ena = '1') then 	timer <= timer + 1;
		else						timer <= (others => '0');
		end if;
	end if;
end process;

----------------------------------------------------------
----------------------------------------------------------	
sm_reg: process (clk, nreset)	
begin
	if(nreset='0')              then CS <= idle;
	elsif(clk'event and clk='1')then CS <= NS;
	end if;
end process; -- sm_reg

sm_combinatorial: 
process (CS, timer, inp)
begin
		timer_ena	<= '0';
		
		case CS is
		when idle	=>
			output		<= '0';			
			if (inp = '1') then 		NS <= wait_for_high;
			else						NS <= idle;
			end if;
			
		when wait_for_high =>
			output 		<= '0';
			timer_ena	<= '1';
			if   (inp = '0') then		NS <= test_low;
			elsif(timer = X"0002") then	NS <= set_high; -- wait 25ns
			else						NS <= wait_for_high;
			end if;
		
		when set_high =>				NS <= input_high;
			output 		<= '1';
			timer_ena	<= '0'; -- important to reset timer here
			
		when input_high =>
			output 		<= '1';
			timer_ena	<= '1';
			if   (timer = X"0028") then	NS <= test_high; -- hold time 500ns
			else						NS <= input_high;
			end if;
		
		when test_high =>
			output 		<= '1';
			if   (inp = '0') then		NS <= wait_for_low;
			else						NS <= test_high;
			end if;
		
		when wait_for_low =>
			output 		<= '1';
			timer_ena	<= '1';
			if   (inp = '1') then		NS <= test_high;
			elsif(timer = X"0002") then	NS <= set_low; -- wait 25ns
			else						NS <= wait_for_low;
			end if;
		
		when set_low =>					NS <= input_low;
			output 		<= '0';
			timer_ena	<= '0'; -- important to reset timer here
			
		when input_low =>
			output 		<= '0';
			timer_ena	<= '1';
			if   (timer = X"0028") then	NS <= test_low; -- hold time 500ns
			else						NS <= input_low;
			end if;
		
		when test_low =>
			output 		<= '0';
			if   (inp = '1') then		NS <= wait_for_high;
			else						NS <= test_low;
			end if;

		when others =>					NS <= idle;
	end case; 
			
end process;

end main;