Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;

-- My Definitions --------------------------------------------------
--USE work.pkg_081006.all;  -- my definitions and entities

-- HOW DOES IT WORK ------------------------------------------------
-- If a signal is detected to be high (on a logic clk edge) then
-- the output is set high for atleast the number of clocks specified
-- by the Filter_Clks input.  If a low is detected after the Filter_Clks
-- have been reached the output is set low for atleast the number of
-- Filter_Clks.


entity UP_DN_Filter_v5 is
	port(clk        	: in std_logic;
		 nreset     	: in std_logic;
		 Input			: in std_logic;
		 Filter_Clks	: in std_logic_vector(15 downto 0);
		 
		 Signal_out		: out std_logic;
		 Up_Pulse		: out std_logic;
		 Dn_Pulse		: out std_logic
		);
end UP_DN_Filter_v5;
	
architecture main of UP_DN_Filter_v5 is

signal inp			: std_logic_vector(1 downto 0);
signal up			: std_logic_vector(1 downto 0);
signal dn			: std_logic_vector(1 downto 0);

signal output		: std_logic;

signal timer		: std_logic_vector(15 downto 0);
signal timer_ena	: std_logic;

	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(idle, hold_high, set_high, hold_low, set_low);
	signal CS, NS  : STATE_TYPE;

-- ## MAIN ##############################################################
begin 

SYNCHRONIZE_INPUT:
process(clk, nreset, Input, inp)
begin
	if   (nreset = '0') then 	inp(0) <= '0';
										inp(1) <= '0';
	elsif rising_edge(clk) then	
										inp(0) <= Input;
										inp(1) <= inp(0);
	end if;	
end process;

PROCESS_TIMER: process(clk, nreset, timer_ena, timer)
begin
	if  (nreset = '0') then 	timer <= (others => '0');
	elsif rising_edge(clk) then
		if  (timer_ena = '1') then 	timer <= timer + 1;
		else									timer <= (others => '0');
		end if;
	end if;
end process;

----------------------------------------------------------
----------------------------------------------------------	
sm_reg: process (clk, nreset)	
begin
	if(nreset='0')              then CS <= idle;
	elsif(clk'event and clk='1')then CS <= NS;
	end if;
end process; -- sm_reg

sm_combinatorial: 
process (CS, timer, inp, Filter_Clks)
begin
		timer_ena	<= '0';
		
		case CS is
		when idle	=>
			output		<= '0';			
			timer_ena	<= '0'; -- important to reset timer here
			
			if (inp(1) = '1') then 		NS <= hold_high;
			else								NS <= idle;
			end if;
		-----------------------------------------------------------------	
		when hold_high =>
			output 		<= '1';
			timer_ena	<= '1';
			
			if(timer > Filter_Clks) then	NS <= set_high;
			else									NS <= hold_high; -- wait here
			end if;
		-----------------------------------------------------------------	
		when set_high =>				
			output 		<= '1';
			timer_ena	<= '0'; -- important to reset timer here
			
			if   (inp(1) = '0') then		NS <= hold_low;
			else									NS <= set_high;
			end if;
		-----------------------------------------------------------------	
		when hold_low =>
			output 		<= '0';
			timer_ena	<= '1';
			
			if(timer > Filter_Clks) then	NS <= set_low; -- hold time 500ns
			else									NS <= hold_low;
			end if;
		-----------------------------------------------------------------	
		when set_low =>					
			output 		<= '0';
			timer_ena	<= '0'; -- important to reset timer here
			
			if   (inp(1) = '1') then		NS <= hold_high;
			else									NS <= set_low;
			end if;
		-----------------------------------------------------------------	
		when others =>					NS <= idle;
	end case; 
			
end process;

UP_PULSE_OUTPUT:
process(clk, nreset, up, output)
begin
	if   (nreset = '0') then 	up(0) <= '0';
										up(1) <= '0';
	elsif rising_edge(clk) then	
										up(0) <= output;
										up(1) <= up(0);
	end if;	
	
	Up_Pulse <= up(0) AND NOT up(1);
end process;

DOWN_PULSE_OUTPUT:
process(clk, nreset, dn, output)
begin
	if   (nreset = '0') then 	dn(0) <= '0';
										dn(1) <= '0';
	elsif rising_edge(clk) then	
										dn(0) <= NOT output;
										dn(1) <= dn(0);
	end if;	
	
	Dn_Pulse <= dn(0) AND NOT dn(1);
end process;

 Signal_out <= output;

end main;