library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
--------------------------------------------------------------------
-- Note:  This module expects 2's complement signed values
--------------------------------------------------------------------

-- My Definitions --------------------------------------------------
USE work.pkg_081006.all;  -- my definitions and entities

entity USB_FIFO_INTERFACE_v4 is
   port(
		clk             : in std_logic;
		nreset          : in std_logic;
		Mem_Data_To_USB	: in std_logic_vector (31 downto 0);				-- Perhaps we should go to 4 bytes as a default
		USB_Data_In     : in std_logic_vector (7 downto 0);                 -- VC++ interface code will need to change also.
		USB_TXE_n		: in std_logic;                                 	-- See state machine %$@$#%@#$%@#$%@$#%@%@#%@#$%@#$%
		USB_RXF_n		: in std_logic;
		mem_32_n16		: in std_logic;
		local_bus_permit: in std_logic;
		-----------------------------------------------------
		USB_Data_Out	: out std_logic_vector (7 downto 0);
		USB_RD_n		: out std_logic;
		USB_WR			: out std_logic;
		DRIVE_DATA_TO_USB  : out std_logic;
		READ_DATA_FROM_USB : out std_logic;
		
		LOWER_Addr_to_Mem : out std_logic_vector (23 downto 0);
		USB_Data_to_Mem : out std_logic_vector (31 downto 0);
		USB_To_Mem_WR	: out std_logic;
		local_bus_request : out std_logic;
		USB_Mode_Out	: out std_logic_vector (7 downto 0); -- Used to determine mode and channel or command
		WD_Tripped		: out std_logic;
		
		test1 : out std_logic;
		test2 : out std_logic
	   );
	   
end USB_FIFO_INTERFACE_v4;
	
architecture main of USB_FIFO_INTERFACE_v4 is
-- Syncronized Inputs -----------------------------------------------------------
signal  sync_rxf_d	: std_logic_vector(1 downto 0);
signal  sync_rxf_q	: std_logic_vector(1 downto 0);
signal	rxf			: std_logic;

signal  sync_txe_d	: std_logic_vector(1 downto 0);
signal  sync_txe_q	: std_logic_vector(1 downto 0);
signal	txe			: std_logic;

-- State machine outputs -----------------------------------------------
signal wr_ena		: std_logic;
signal drive_data	: std_logic;
signal xfr_done		: std_logic;

-- USB Data In Buffer Variables ----------------------------------------
signal USB_In_Buffer	: std_logic_vector(7 downto 0);
signal USB_Data			: std_logic_vector(7 downto 0);

-- USB Header Buffers --------------------------------------------------
signal hdr_0		: std_logic_vector(7 downto 0); -- mode | address[20...16]
signal hdr_1		: std_logic_vector(7 downto 0); -- address[15...8]
signal hdr_2		: std_logic_vector(7 downto 0); -- address[ 7...0]
signal hdr_3		: std_logic_vector(7 downto 0); -- bytecnt[15...8]
signal hdr_4		: std_logic_vector(7 downto 0); -- bytecnt[ 7...0]
signal hdr_cnt		: std_logic_vector(2 downto 0);
signal inc_hdr_cnt  : std_logic;
signal rst_hdr_cnt	: std_logic;

-- USB Data Byte Buffers ----------------------------------------------
signal byte_0		: std_logic_vector(7 downto 0); 
signal byte_1		: std_logic_vector(7 downto 0); 
signal byte_2		: std_logic_vector(7 downto 0); 
signal byte_3		: std_logic_vector(7 downto 0); 

-- Address Pointer ----------------------------------------------------
signal first_addr	: std_logic_vector(20 downto 0);
signal addr_ptr 	: std_logic_vector(20 downto 0);
signal set_addr_ptr	: std_logic;
signal inc_addr_ptr	: std_logic;

-- Byte Count Counter -------------------------------------------------
signal num_bytes	: std_logic_vector(15 downto 0);
signal byte_count	: std_logic_vector(15 downto 0);
signal rst_byte_cnt	: std_logic;
signal inc_byte_cnt : std_logic;

-- Protocol Reset Timer -----------------------------------------------
signal ptimer		: std_logic_vector(31 downto 0);
signal rst_ptimer	: std_logic;
signal timed_out	: std_logic;

-- Data Control Signals -----------------------------------------------
signal get_data		: std_logic;
signal get_hdr		: std_logic;
signal get_byte		: std_logic;

signal rd_usb		: std_logic;
signal wr_usb		: std_logic;

-- Data Buffer Variables -----------------------------------------------
signal Data_Buffer_out	: std_logic_vector(7 downto 0);
signal Data_Buffer_ena	: std_logic;

-- Data Setup Delay Counter -------------------------------------------
signal  delay_ena 		: std_logic;
signal	delay_count		: std_logic_vector(3 downto 0);
signal	delay_done		: std_logic;
 
-- Local Bus Request Latch --------------------------------------------
signal	set_local_bus_req : std_logic;
signal	clr_local_bus_req : std_logic;
signal	bus_req_latch	  : std_logic;

signal  latch_data_to_USB : std_logic;
signal  Data_To_USB		  : std_logic_vector(31 downto 0);

-- Operations State Machine signals -----------------------------------
	type STATE_TYPE is 
	(idle, rd_header, wait1, eval_header, full_addr_mode, 
		wait_for_next_byte, get_next_hdr_byte, wait2, setup_transfer,
		do_read_loop, setup_mem_data, latch_mem_data, check_usb_txe,
		get_mem_data, set_next_address, next_read_loop, read_done, do_write_loop, 
		get_byte_in, wait3, write_buffer_to_mem, write_to_mem_dly1, write_to_mem_dly2, 
		next_address, get_next_byte, write_done, chan_read, chan_write, 
		cmd_mode, done);
	signal CS, NS  : STATE_TYPE;

	type STATE_TYPE2 is 
		(idle2, rd, get_usb_byte, wait_rd, wr, finish_wr, done2);
	signal CS2, NS2  : STATE_TYPE2;

--========================================================================
begin

HEADER_BUFFERS: 
process(clk, nreset, USB_Data_In, get_data, get_hdr, hdr_cnt,
		hdr_0, hdr_1, hdr_2, hdr_3, hdr_4)
begin
	if   (nreset = '0') then
		hdr_0 <= (others => '0');
		hdr_1 <= (others => '0');
		hdr_2 <= (others => '0');
		hdr_3 <= (others => '0');
		hdr_4 <= (others => '0');
	elsif(rising_edge(clk) AND (get_data = '1') AND (get_hdr = '1'))then
		case hdr_cnt is
			when "000" => hdr_0 <= USB_Data_In;
			when "001" => hdr_1 <= USB_Data_In;
			when "010" => hdr_2 <= USB_Data_In;
			when "011" => hdr_3 <= USB_Data_In;
			when "100" => hdr_4 <= USB_Data_In;	
			when others => null;	
		end case;		
	end if;	
	
end process;	
----------------------------------------------------------	
	first_addr 		<= hdr_0(4 downto 0) & hdr_1 & hdr_2;
	USB_Mode_Out	<= hdr_0(7 downto 0);
----------------------------------------------------------

DETERMINE_NUMBER_OF_BYTES:
process(hdr_0(6), mem_32_n16, hdr_3, hdr_4)
begin
	if (hdr_0(6) = '1') then 	 --channel mode
		if  (mem_32_n16 = '1') then num_bytes <= X"0004";
		else						num_bytes <= X"0002";
		end if;
	else 						num_bytes <= hdr_3 & hdr_4;
	end if;
end process;
----------------------------------------------------------

DATA_BUFFERS: 
process(clk, nreset, USB_Data_In, get_data, get_byte, hdr_cnt, mem_32_n16)
begin
	if   (nreset = '0') then
		byte_0 <= (others => '0');
		byte_1 <= (others => '0');
		byte_2 <= (others => '0');
		byte_3 <= (others => '0');
	elsif(rising_edge(clk) AND (get_data = '1') AND (get_byte = '1'))then
		case byte_count(1 downto 0) is
			when "00" => byte_0 <= USB_Data_In;
			when "01" => byte_1 <= USB_Data_In;
			when "10" => 
				if  (mem_32_n16 = '1') then byte_2 <= USB_Data_In;
				else						byte_0 <= USB_Data_In;
				end if;
			when "11" => 
				if  (mem_32_n16 = '1') then byte_3 <= USB_Data_In;
				else						byte_1 <= USB_Data_In;
				end if;
		end case;		 
	end if;	
	
end process;	

HEADER_POINTER:		
process(clk, nreset, inc_hdr_cnt, rst_hdr_cnt)
begin
	if   (nreset = '0') then
		hdr_cnt <= "000";
	elsif rising_edge(clk) then
		if   (inc_hdr_cnt = '1') then	hdr_cnt <= hdr_cnt + 1;
		elsif(rst_hdr_cnt = '1') then	hdr_cnt <= "000";
		else							hdr_cnt <= hdr_cnt;
		end if;
	end if;	
end process;		

---------------------------------------------------------------
ADDRESS_POINTER:
-- RECALL: first_addr <= hdr_0(4 downto 0) & hdr_1 & hdr_2;

process(clk, nreset, set_addr_ptr, inc_addr_ptr, first_addr)
begin
	if   (nreset = '0') then
		addr_ptr <= (others => '0');
	elsif rising_edge(clk) then		
		if   (inc_addr_ptr = '1') then  addr_ptr <= addr_ptr + 2;	
		elsif(set_addr_ptr = '1') then	addr_ptr <= first_addr;
		else							addr_ptr <= addr_ptr;
		end if;
	end if;	
end process;		

BYTES_TRANSFERED_COUNT:
process(clk, nreset, rst_byte_cnt, inc_byte_cnt)
begin
	if   (nreset = '0') then
		byte_count <= (others => '0');
	elsif rising_edge(clk) then
		if   (inc_byte_cnt = '1') then	byte_count <= byte_count + 1;
		elsif(rst_byte_cnt = '1') then	byte_count <= (others => '0');
		else							byte_count <= byte_count;
		end if;
	end if;	
end process;		

PROTOCOL_RESET_TIMER:
process(clk, nreset,rst_ptimer, ptimer)
begin
	if(nreset = '0') then ptimer <= (others => '0');
	elsif rising_edge(clk) then
		if   (rst_ptimer = '1') then ptimer <= (others => '0');
		else						 ptimer <= ptimer + 1;
		end if;
	
		if (ptimer > X"05f5e100") then timed_out <= '1';
		else						   timed_out <= '0';
		end if;
	end if;
end process;

--------------------------------------------------------------------------			
-- Syncronization of USB FIFO control signals
SYNC_RXF: dffr 
		generic map(2)
		port map(	d    => sync_rxf_d(1 downto 0),
					clk  => clk, clrn => nreset, ena  => '1', prn  => '1',
					q    => sync_rxf_q(1 downto 0)
				); 

			sync_rxf_d(0) <= NOT USB_RXF_n;
			sync_rxf_d(1) <= sync_rxf_q(0);
			rxf		 	  <= sync_rxf_q(1);	
					
--------------------------------------------------------------------------			
SYNC_TXE: dffr
		generic map(2)
		port map(	d    => sync_txe_d(1 downto 0),
					clk  => clk, clrn => nreset, ena  => '1', prn  => '1',
					q    => sync_txe_q(1 downto 0)
				); 

			sync_txe_d(0) <= NOT USB_TXE_n;
			sync_txe_d(1) <= sync_txe_q(0);
			txe		 	  <= sync_txe_q(1);	

--========================================================================
LATCH_LOCAL_BUS_REQUEST:
process(clk, nreset, set_local_bus_req, clr_local_bus_req)
begin
	if   (nreset = '0') then bus_req_latch <= '0';
	elsif rising_edge(clk) then
		if   (set_local_bus_req = '1') then	bus_req_latch <= '1';
		elsif(clr_local_bus_req = '1') then bus_req_latch <= '0';
		else								bus_req_latch <= bus_req_latch;
		end if;
	end if;
end process;

local_bus_request <= bus_req_latch;

--========================================================================	
-- Data Setup Delay Timer
		
DELAY_TIMER: counter 
generic map(4)
port map(	clk  => clk,
			clrn => delay_ena, -- resets when not enabled
			ena  => delay_ena,
			load => '0',
			updn => '1',
			ld_data  => (others => '0'),
			flag_cnt => delay_count,
			flag_out => delay_done,
			cnt_out  => open        );

--=========================================================================
-- State Machine Process definition ---------------------------
	sm_reg: process (clk, nreset, timed_out)	
	begin
		if(nreset='0')OR(timed_out = '1') then CS <= idle;
		elsif(clk'event and clk='1')then CS <= NS;
		end if;
	end process; -- sm_reg
	
--	type STATE_TYPE is 
--	(idle, rd_header, wait1, eval_header, full_addr_mode, 
--		wait_for_next_byte, get_next_hdr_byte, wait2, setup_transfer,
--		do_read_loop, get_mem_data, set_next_address, setup_mem_data,
--		next_read_loop, read_done, chan_read, chan_write, cmd_mode,
--		done);

	sm_next_state: 
	process (CS, rxf, xfr_done, hdr_0, hdr_cnt, Mem_Data_To_USB, num_bytes,  
			local_bus_permit, byte_count, byte_3, byte_2, byte_1, byte_0, txe)
	begin
		-- default values ---------------------------
		rst_ptimer			<= '0';
		inc_hdr_cnt 		<= '0';
		rst_hdr_cnt 		<= '0';
		
		rst_byte_cnt 		<= '0';
		inc_byte_cnt		<= '0';
		
		set_addr_ptr 		<= '0';
		inc_addr_ptr		<= '0';
		
		get_hdr				<= '0';
		get_byte			<= '0';
		
		rd_usb				<= '0';
		wr_usb				<= '0';
		set_local_bus_req 	<= '0';
		clr_local_bus_req 	<= '0';
--		USB_Data_Out 		<= Mem_Data_To_USB( 7 downto 0);
		USB_Data_to_Mem 	<= byte_3 & byte_2 & byte_1 & byte_0; -- Data Buffer
		USB_To_Mem_WR		<= '0';
		
		latch_data_to_USB 	<= '0';


		---------------------------------------------
		case CS is
			when idle		=> 	
				rst_hdr_cnt <= '1';
				rst_ptimer	<= '1'; -- reset the protocol timer (watchdog timer)
				clr_local_bus_req 	<= '1';
				
				if(rxf = '1')		then 		NS <= rd_header;
				else                      		NS <= idle;
				end if;
			---------------------------------------------
			when rd_header 	=>					NS <= wait1;
				get_hdr		<= '1';
				rd_usb		<= '1';
			
			when wait1 =>	
				get_hdr		<= '1';
				if (xfr_done = '1') then  		NS <= eval_header;
				else							NS <= wait1; -- wait
				end if;

			when eval_header =>	
				case hdr_0(7 downto 5) is
					when "000" =>				NS <= full_addr_mode;
					when "001" =>				NS <= full_addr_mode;
					when "010" =>				NS <= chan_read;
					when "011" =>				NS <= chan_write;
					when "100" =>				NS <= cmd_mode;
					when others =>				NS <= done;              
				end case;
			
			--------------------------------------------------------------------------
			-- start full address mode loop ------------------------------------------
			-- Loop here to get the other 4 header bytes from USB
			when full_addr_mode =>		
				if   (hdr_cnt = "100") then		NS <= setup_transfer;
				else
					inc_hdr_cnt <= '1';			NS <= wait_for_next_byte;
				end if;
			
			when wait_for_next_byte =>
				if(rxf = '1')		then 		NS <= get_next_hdr_byte;
				else                      		NS <= wait_for_next_byte;
				end if;
	
			when get_next_hdr_byte =>			NS <= wait2;
				get_hdr		<= '1';
				rd_usb		<= '1';
				
			when wait2 =>	
				get_hdr		<= '1';
				if (xfr_done = '1') then  		NS <= full_addr_mode;
				else							NS <= wait2; -- wait
				end if;
			-- end read header loop ---------------------------------------------			
			---------------------------------------------------------
			when setup_transfer =>
				rst_byte_cnt <= '1';
				set_addr_ptr <= '1';
				
				set_local_bus_req 	<= '1';
				
				if   (local_bus_permit = '1') then
					if   (hdr_0(5) = '0') then	NS <= do_read_loop;
					else						NS <= do_write_loop;
					end if;
				else							NS <= setup_transfer;--wait
				end if;
			
			-----------------------------------------------------------------------
			-- start of read channel ----------------------------------------------
			-- NOTE: USB_Mode_Out has been set so data mux is routing the selected
			--       channel to the Mem_Data_To_USB input.  The num_bytes varialble
			--       has been set to 4 by the hdr_0(6) bit being '1'.
			when chan_read =>			  		NS <= do_read_loop;
				rst_byte_cnt <= '1';
				set_addr_ptr <= '1'; -- the address pointer will be ignored in this case
									 -- there is no local_bus_request made in this mode
									 -- so the address and data lines out of this module
									 -- will not be connected to the local bus.
			
			-- end of read channel ------------------------------------------------
			-----------------------------------------------------------------------

			-----------------------------------------------------------------------
			-- start read loop ----------------------------------------------------
			when do_read_loop =>				NS <= setup_mem_data;

			when setup_mem_data =>				NS <= latch_mem_data;
						
			when latch_mem_data =>				NS <= check_usb_txe;
				latch_data_to_USB 	<= '1';

			when check_usb_txe =>
				if(byte_count = num_bytes) then	NS <= read_done;
				else							
					if(txe = '1') then 			NS <= get_mem_data;
					else						NS <= check_usb_txe; --wait
					end if;
				end if;
			
			when get_mem_data =>
				wr_usb <= '1';
				if 	 (xfr_done = '0')      then NS <= get_mem_data; --wait
				elsif(byte_count(0) = '1') then	NS <= set_next_address;	
						--^-- this is set to transfer 2 bytes before advancing the pointer by 2 byte addresses
				else							NS <= next_read_loop;
				end if;
				
			when set_next_address =>			NS <= do_read_loop;
				inc_addr_ptr <= '1';
				inc_byte_cnt <= '1';
				rst_ptimer	<= '1'; --<<<<<<<<< reset ptimer
				
			when next_read_loop =>				NS <= check_usb_txe;
				inc_byte_cnt <= '1';
				rst_ptimer	<= '1'; --<<<<<<<<< reset ptimer

			-- end of read loop ---------------------------------------------------
			when read_done =>			  		NS <= idle;
				clr_local_bus_req 	<= '1';
			-----------------------------------------------------------------------
			
			----------------------------------------------------------------------
			-- start write loop --------------------------------------------------
			when do_write_loop =>		  		
				if(byte_count = num_bytes) then	NS <= write_done;
				else							
					if(rxf = '1') then 			NS <= get_byte_in;
					else						NS <= do_write_loop; --wait
					end if;
				end if;
			
			when get_byte_in =>					NS <= wait3;
				rd_usb		<= '1'; -- This "calls" the other SM to read the usb fifo
				get_byte	<= '1'; -- This directs the usb fifo byte into the Data Buffer
				
			when wait3 =>	
				get_byte	<= '1';
				if (xfr_done = '1') then  		
					if (byte_count(0) = '1') then NS <= write_buffer_to_mem;												
					else						  NS <= get_next_byte;
					end if;
				else							NS <= wait3; -- wait
				end if;
		
			when write_buffer_to_mem =>			NS <= write_to_mem_dly1;
				USB_Data_to_Mem <= byte_3 & byte_2 & byte_1 & byte_0;
				USB_To_Mem_WR	<= '1';
			
			when write_to_mem_dly1 =>			NS <= write_to_mem_dly2;
				USB_Data_to_Mem <= byte_3 & byte_2 & byte_1 & byte_0;
				USB_To_Mem_WR	<= '1';
			
			when write_to_mem_dly2 =>			NS <= next_address;
				USB_Data_to_Mem <= byte_3 & byte_2 & byte_1 & byte_0;
				USB_To_Mem_WR	<= '0';
			
			when next_address =>				NS <= get_next_byte;
				inc_addr_ptr <= '1';
				
			when get_next_byte =>				NS <= do_write_loop;
				inc_byte_cnt <= '1';
				rst_ptimer	<= '1'; --<<<<<<<<< reset ptimer
								
			-- end of write loop ---------------------------------------------------
			when write_done =>			  		NS <= idle;
				clr_local_bus_req 	<= '1';
			-----------------------------------------------------------------------
			
			when chan_write =>			  		NS <= idle;
			when cmd_mode =>			  		NS <= idle;
			when done =>			  			NS <= idle;        
			
			when others  =>				  		NS <= idle;
		end case;
	end process;


--=========================================================================
-- State Machine Process definition for Transfer Timing -------------------
	sm_reg2: process (clk, nreset, timed_out)	
	begin
		if(nreset='0')OR(timed_out = '1') then CS2 <= idle2;
		elsif(clk'event and clk='1')then CS2 <= NS2;
		end if;
	end process; -- sm_reg
	
--	type STATE_TYPE2 is 
--		(idle2, rd, get_usb_byte, wait_rd, wr, finish_wr, done2);

	sm_next_state2: process (CS2, rd_usb, wr_usb, rxf, txe, delay_done)
	begin
		-- default values ---------------------------
		USB_RD_n	<= '1';
		READ_DATA_FROM_USB	<= '0';
		
		get_data	<= '0';
		delay_ena  	<= '0';
		delay_count <= "0111";
		xfr_done 	<= '0';
		wr_ena 		<= '0';
		drive_data  <= '0';
		---------------------------------------------
		case CS2 is
			when idle2 => 			  				
				if   (rd_usb = '1') then	NS2 <= rd;
				elsif(wr_usb = '1') then	NS2 <= wr;
				else                      	NS2 <= idle2;
				end if;
				
			----------------------------------------------------
			when rd =>				
				USB_RD_n	<= '0';
				READ_DATA_FROM_USB	<= '1';	
							
				delay_ena  	<= '1';				
				if (delay_done = '1') then  NS2 <= get_usb_byte;
				else						NS2 <= rd; -- wait
				end if;
			
			when get_usb_byte =>			NS2 <= wait_rd;
				get_data 	<= '1';
				
			when wait_rd =>				
				if   (rxf = '0')	then 	NS2 <= done2;
				else                      	NS2 <= wait_rd;
				end if;
				
			----------------------------------------------------
			when wr =>
				wr_ena 		<= '1';
				drive_data  <= '1';
				
				delay_ena  	<= '1';				
				if (delay_done = '1') then  NS2 <= finish_wr;
				else						NS2 <= wr; -- wait
				end if;

			when finish_wr =>				
				drive_data <= '1';
				
				if   (txe = '0')	then 	NS2 <= done2;
				else                      	NS2 <= finish_wr; --wait
				end if;

			----------------------------------------------------
			when done2 =>					NS2 <= idle2;
				xfr_done <= '1';
				
			when others =>					NS2 <= idle2;
			
		end case;
	end process;
	
	-------------------------------------------------------------------------
	DATA_TO_USB_BUFFER:
	process (clk, nreset, Mem_Data_To_USB, latch_data_to_USB)
	begin
		if   (nreset = '0') then Data_to_USB <= (others=>'0');
		elsif rising_edge(clk) then
			if(latch_data_to_USB = '1') then 	Data_to_USB <= Mem_Data_To_USB;
			else								Data_to_USB <= Data_To_USB;
			end if;
		end if;
	end process;
	
	-------------------------------------------------------------------------
	SET_BYTES_OF_USB_DATA_OUT:
	process (byte_count, Data_To_USB, mem_32_n16)
	begin
		case byte_count(1 downto 0) is
			when "00" => USB_Data_Out <= Data_To_USB( 7 downto 0);
			when "01" => USB_Data_Out <= Data_To_USB(15 downto 8);
			when "10" => 
				if  (mem_32_n16 = '1') then
					USB_Data_Out <= Data_To_USB(23 downto 16);
				else
					USB_Data_Out <= Data_To_USB( 7 downto 0);
				end if;
			when "11" => 
				if  (mem_32_n16 = '1') then
					USB_Data_Out <= Data_To_USB(31 downto 24);
				else
					USB_Data_Out <= Data_To_USB(15 downto 8);
				end if;
		end case;
	end process;
	--------------------------------------------------------------------------
	process (clk, nreset)
	begin
		if (nreset = '0') then
			USB_WR				<= '0';
			DRIVE_DATA_TO_USB	<= '0';
		elsif (rising_edge(clk)) then
			USB_WR				<= wr_ena;
			DRIVE_DATA_TO_USB	<= drive_data;
		end if;
	end process;
	
	LOWER_Addr_to_Mem 	<= "000" & addr_ptr; 
	
	WD_Tripped <= timed_out;
	test1 <= inc_addr_ptr;	
	test2 <= set_addr_ptr;	

end;